# -*- coding: utf-8 -*-

from datetime import date

from baseapp import BaseApp
from enum import *
from emc import Group, DataSource, User


class Gene(dict):
    """ Structure for (aggregation) gene

    Fields:
        id (str): The id of the gene. It should be unique in a subcategory
        name (str); Gene name
        timerange (int): Gene time range
        type (str): Gene type, refer to GeneType for valid values
        chartType (str): Gene chart type, refer to GeneChartType for valid values
        uiInfo (obj): Gene UI info
    """

    def __init__(self, id, name, time_range, type, chart_type, ui_info):
        assert id and name and time_range is not None and type and chart_type and ui_info
        super(Gene, self).__init__({
            'id': id,
            'name': name,
            'timerange': time_range,
            'type': type,
            'chartType': chart_type,
            'uiInfo': ui_info if isinstance(ui_info, GeneUIInfo) else GeneUIInfo.from_dict(ui_info)
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['id'], dict_obj['name'], dict_obj['timerange'], dict_obj['type'], dict_obj['chartType'],
                   dict_obj['uiInfo'])


class GeneCategory(dict):
    """ Structure for (aggregation) gene category

    Fields:
        name (str): Fixed gene name
        subcategories (list): A list of FixedGeneCategory as subcategories
        genes (list): A list of

        id (int): The auto id
    """

    def __init__(self, name, subcategories, genes, id=None):
        assert name and subcategories is not None and isinstance(subcategories,
                                                                 list) and genes is not None and isinstance(genes, list)
        super(GeneCategory, self).__init__({
            'name': name,
            'subcategories': [x if isinstance(x, FixedGeneCategory) else FixedGeneCategory.from_dict(x) for x in
                              subcategories],
            'genes': [x if isinstance(x, FixedGene) else FixedGene.from_dict(x) for x in genes],
            'id': id
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['name'], dict_obj['subcategories'], dict_obj['genes'], dict_obj.get('id'))


class BandGene(dict):
    """ Structure for a gene-based band

    Fields:
        geneId (str): The gene id
        cid (int): The data source to calculate the band
        operator (str): The band operator to calculate the band
        operand (str): The operands to calculate the band
    """

    def __init__(self, gene_id, data_source, operator, operand):
        assert gene_id and data_source and operator and operand
        super(BandGene, self).__init__({
            'geneId': gene_id,
            'cid': data_source['id'] if isinstance(data_source, DataSource) else data_source,
            'operator': operator,
            'operand': operand
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['geneId'], dict_obj['cid'], dict_obj['operator'], dict_obj['operand'])


class BandCombine(dict):
    """ Structure for a combined band

    Fields:
        bandIds (list): List of band ids to combine
        operators (list): List of operators using to combine the bands (only the first element is used yet)
    """

    def __init__(self, bands, operators):
        assert bands and operators
        super(BandCombine, self).__init__({
            'bandIds': [x['id'] if isinstance(x, Band) else x for x in bands],
            'operators': operators
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['bandIds'], dict_obj['operators'])


class Band(dict):
    """ Structure for a band
    
    Fields:
        categoryId (int): The band category of the band belongs to
        name (str): Band name
        description (str): Band description
        type (str): Band type, refer to 'BandType' for valid values
        targetGene (obj): Gene setting if type=gene or type=fixedgene
        targetBand (obj): Combine bands if type=combine

        needRefresh (bool): Need to refresh the band or not
        snapshotInfo (obj): Snapshot info (if a snap-shot band), or None for pure band

        id (int): The auto id
        amount (int): Amount of users in the band
        updateTime (long): The band update time in Epoch (milliseconds)
    """

    def __init__(self, category, name, description,
                 type, target_gene=None, target_band=None,
                 need_refresh=True, snapshot_info=None,
                 id=None, amount=None, update_time=None):
        super(Band, self).__init__({
            'categoryId': category['id'] if isinstance(category, BandCategory) else category,
            'name': name,
            'description': description,
            'needRefresh': need_refresh,
            'type': type,
            'targetGene': target_gene,
            'targetBand': target_band,
            'snapshotInfo': snapshot_info,
            'id': id,
            'amount': amount,
            'updateTime': update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['categoryId'], dict_obj['name'], dict_obj['description'],
                   dict_obj['type'], dict_obj.get('targetGene'), dict_obj.get('targetBand'),
                   dict_obj['needRefresh'], dict_obj.get('snapshotInfo'),
                   dict_obj.get('id'), dict_obj.get('amount'), dict_obj.get('updateTime'))

    @classmethod
    def from_dict_with_ext_cat(cls, dict_obj, category):
        assert dict_obj and category
        category_id = category['id'] if isinstance(category, BandCategory) else int(category)
        return cls(category_id, dict_obj['name'], dict_obj['description'],
                   dict_obj['type'], dict_obj.get('targetGene'), dict_obj.get('targetBand'),
                   dict_obj['needRefresh'], dict_obj.get('snapshotInfo'),
                   dict_obj.get('id'), dict_obj.get('amount'), dict_obj.get('updateTime'))


class GeneUIInfo(dict):
    """ Structure for gene UI info

    Fields:
        Xaxis_id (str): The string id of gene chart x-axis
        Yaxis_id (str): The string id of gene chart y-axis
        unit (str): The string id of gene unit (used on gene filter UI, etc.)
        Xaxis (str): The (localized) string of gene chart x-axis directly set by system operator
        Xaxis (str): The (localized) string of gene chart y-axis directly set by system operator
    """

    def __init__(self, x_axis_id=None, y_axis_id=None, unit=None, x_axis=None, y_axis=None):
        super(GeneUIInfo, self).__init__({
            'Xaxis_id': x_axis_id,
            'Yaxis_id': y_axis_id,
            'unit': unit,
            'Xaxis': x_axis,
            'Yaxis': y_axis
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj.get('Xaxis_id'), dict_obj.get('Yaxis_id'), dict_obj.get('unit'),
                   dict_obj.get('Xaxis'), dict_obj.get('Yaxis'))


class FixedGene(dict):
    """ Structure for fixed (external) gene

    Fields:
        id (str): The id of the fixed gene. It should be unique in a fixed subcategory
        name (str); Gene name
        timerange (int): Gene time range
        type (str): Gene type, refer to GeneType for valid values
        chartType (str): Gene chart type, refer to GeneChartType for valid values
        uiInfo (obj): Gene UI info
    """

    def __init__(self, id, name, time_range, type, chart_type, ui_info):
        assert id and name and time_range is not None and type and chart_type and ui_info
        super(FixedGene, self).__init__({
            'id': id,
            'name': name,
            'timerange': time_range,
            'type': type,
            'chartType': chart_type,
            'uiInfo': ui_info if isinstance(ui_info, GeneUIInfo) else GeneUIInfo.from_dict(ui_info)
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['id'], dict_obj['name'], dict_obj['timerange'], dict_obj['type'], dict_obj['chartType'],
                   dict_obj['uiInfo'])


class FixedGeneCategory(dict):
    """ Structure for fixed (external) gene categories

    Fields:
        name (str): Fixed gene name
        subcategories (list): A list of FixedGeneCategory as subcategories
        genes (list): A list of

        id (int): The auto id
    """

    def __init__(self, name, subcategories, genes, id=None):
        assert name and subcategories is not None and isinstance(subcategories,
                                                                 list) and genes is not None and isinstance(genes, list)
        super(FixedGeneCategory, self).__init__({
            'name': name,
            'subcategories': [x if isinstance(x, FixedGeneCategory) else FixedGeneCategory.from_dict(x) for x in
                              subcategories],
            'genes': [x if isinstance(x, FixedGene) else FixedGene.from_dict(x) for x in genes],
            'id': id
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['name'], dict_obj['subcategories'], dict_obj['genes'], dict_obj.get('id'))


class BandCategory(dict):
    """ Structure for a band category

    Fields:
        name (str): The band category name
        bands (list): A list of Bands
        id (int): The auto id
    """

    def __init__(self, name, bands=None, id=None):
        assert name
        if bands is None:
            bands = []
        assert isinstance(bands, list)
        super(BandCategory, self).__init__({
            'name': name,
            'bands': [x if isinstance(x, Band) else Band.from_dict_with_ext_cat(x, id) for x in bands],
            'id': id
        })

    def to_simple(self):
        return {'name': self['name']}

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict_obj.get('bands'), dict_obj.get('id'))


class PopulationSummary(dict):
    """ Structure for population summary item

    Fields:
        key (str): The key value of the summary item (depending on the gene)
        amount (int): The segment size with the key value
    """

    def __init__(self, key, amount):
        super(PopulationSummary, self).__init__({
            'key': key,
            'amount': amount
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['key'], dict_obj['amount'])


class EIGroup(dict):
    """ Structure for Etu Insight group

    Fields:
        name (str): The group id/name
        displayName (str): The display name

        id (int): The auto id (the same as EMC group)
    """

    def __init__(self, name, display_name, id=None):
        super(EIGroup, self).__init__({
            'name': name,
            'displayName': display_name,
            'id': id
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['name'], dict_obj['displayName'], dict_obj.get('id'))

    @classmethod
    def from_emc_group(cls, emc_group):
        assert emc_group and isinstance(emc_group, Group)
        return cls(emc_group['name'], emc_group['displayName'], emc_group['id'])


class EIUser(dict):
    """ Structure for Etu Insight user

    Fields:
        name (str): The login name of the user
        displayName (str): The display name
        role (str): The user role in Etu Insight, refer to AppRoleName for valid values.
                    It's a single role not a list, as the roles in EMC User object
        group (object): The group info of the user belongs to.

        id (int): The auto id (the same as EMC user)
    """

    def __init__(self, name, display_name, role, group, id=None):
        super(EIUser, self).__init__({
            'name': name,
            'displayName': display_name,
            'role': role,
            'group': group if isinstance(group, EIGroup) else
                     EIGroup.from_emc_group(group) if isinstance(group, Group) else EIGroup.from_dict(group),
            'id': id
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['name'], dict_obj['displayName'], dict_obj['role'], dict_obj['group'], dict_obj.get('id'))

    @classmethod
    def from_emc_user(cls, emc_group, emc_user):
        assert emc_group
        assert emc_user and isinstance(emc_user, User)
        filtered_role = filter(lambda u: u['appId'] == AppId.EI, emc_user.roles)
        if len(filtered_role) == 0:
            raise RuntimeError('The user (%s) does not have EI authorization.' % emc_user['name'])

        ei_role = filtered_role[0]
        return cls(emc_user['name'], emc_user['displayName'], ei_role['roleName'], emc_group, emc_user['id'])


class EI3(BaseApp):
    """ Encapsulate Etu Insight (v3) API """

    __APP_NAME = 'EI3'
    __HOST = 'eihome.online.etunexus.com'
    __API_BASE = '/EI/V3/service/v1'
    __SHIRO_CAS_BASE = '/EI/V3/shiro-cas'

    def __init__(self, cas, host=None, api_base=None, shiro_cas_base=None):
        """ Constructor """

        api_base = api_base if api_base else self.__API_BASE
        super(EI3, self).__init__(cas, EI3.__APP_NAME,
                                  api_host=host if host else self.__HOST,
                                  api_base=api_base if api_base else self.__API_BASE,
                                  shiro_cas_base=shiro_cas_base if shiro_cas_base else self.__SHIRO_CAS_BASE)

    # User info
    def get_me(self):
        res = self.request_get('/user/me')
        return EIUser.from_dict(res['data'])

    def get_superme(self):
        res = self.request_get('/user/superme')
        return EIUser.from_dict(res['data'])

    # Gene category and detail gene info
    def get_gene_categories(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/genecategory?groupId={0}'.format(group_id))
        return [GeneCategory.from_dict(x) for x in res['data']]

    # Band category
    def get_band_categories(self):
        res = self.request_get('/bandcategory')
        return [BandCategory.from_dict(x) for x in res['data']]

    def add_band_category(self, band_category):
        assert band_category and isinstance(band_category, BandCategory)
        res = self.request_post_form('/bandcategory', band_category.to_simple())
        return BandCategory.from_dict(res['data'])

    def update_band_category(self, band_category):
        assert band_category and isinstance(band_category, BandCategory)
        band_category_id = band_category['id']
        assert band_category_id
        res = self.request_post_form('/bandcategory/{0}'.format(band_category_id), band_category.to_simple())
        return BandCategory.from_dict(res['data'])

    def del_band_category(self, band_category):
        assert band_category
        band_category_id = band_category['id'] if isinstance(band_category, BandCategory) else band_category
        assert band_category_id
        res = self.request_del('/bandcategory/{0}'.format(band_category_id))
        return res['data']

    # Band
    def add_band(self, band, file_path=None):
        assert band and isinstance(band, Band)
        band_category_id = band['categoryId']
        if file_path:
            assert band['type'] == BandType.UPLOAD
            res = self.request_upload('/band', band, file_path)
        else:
            res = self.request_post_multipart('/band', band)
        return Band.from_dict_with_ext_cat(res['data'], band_category_id)

    def update_band(self, band, file_path=None):
        assert band and isinstance(band, Band)
        band_category_id = band['categoryId']
        band_id = band['id']
        assert band_id
        if file_path:
            assert band['type'] == BandType.UPLOAD
            res = self.request_upload('/band/{0}'.format(band_id), band, file_path)
        else:
            res = self.request_post_multipart('/band/{0}'.format(band_id), band)
        return Band.from_dict_with_ext_cat(res['data'], band_category_id)

    def del_band(self, band):
        assert band
        band_id = band['id'] if isinstance(band, Band) else band
        assert band_id
        res = self.request_del('/band/{0}'.format(band_id))
        return res['data']

    # Fixed (external) gene
    def get_fixed_gene_categories(self):
        res = self.request_get('/fixedgenecategory')
        return [FixedGeneCategory.from_dict(x) for x in res['data']]

    def upload_fixed_gene_schema(self, group, file_path):
        """ Upload fixed (external) gene schema (admin/operator only) """
        assert group and file_path
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_upload('/fixedgene/schema', {'groupId': group_id}, file_path)
        return FixedGeneCategory.from_dict(res['data'])

    def upload_fixed_gene_data(self, group, file_path):
        """ Upload fixed (external) gene data (admin/operator only) """
        assert group and file_path
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_upload('/fixedgene/data', {'groupId': group_id}, file_path)
        return res['data']

    # Population summary data
    def get_population_summary(self, gene, data_source):
        """ Get population summary data """
        assert gene and data_source
        gene_id = gene['id'] if isinstance(gene, Gene) else int(gene)
        data_source_id = data_source['id'] if isinstance(data_source, DataSource) else int(data_source)
        res = self.request_get('/population/summary?geneId={0}&cId={1}'.format(gene_id, data_source_id))
        return [PopulationSummary.from_dict(x) for x in res['data']]

    # TODO: Add support for population timeline API

    # Statistics
    def get_statistics(self, group, items, start_date, end_date=date.today()):
        """ Get EI statistics.

        Arguments:
            group (obj or int): A Group instance or group id
            items (list): Refer to EIStatisticsItem for valid values, and put the (multiple) items into a list.
                e.g. [EIStatisticsItem.DOWNLOAD_UID_LIST, EIStatisticsItem.ADD_BAND]
            start_date (date or str): Start date of the statistics to query. It could be a datetime.date instance, or a
                string in 'yyyy-mm-dd' format
            end_date (date or str): End date of the statistics to query. It could be a datetime.date instance, or a
                string in 'yyyy-mm-dd' format

        Return:
            A list of tuple:
             [0] A date string in yyyy-mm-dd
             [1] An item:count dictionary

            The result is sorted by the date.
        """
        assert group and items and isinstance(items, list) and start_date and end_date
        group_id = group['id'] if isinstance(group, Group) else int(group)
        start_date_str = '%04d-%02d-%02d' % (start_date.year, start_date.month, start_date.day) \
            if isinstance(start_date, date) else str(start_date)
        end_date_str = '%04d-%02d-%02d' % (end_date.year, end_date.month, end_date.day) \
            if isinstance(end_date, date) else str(end_date)
        item_str = ','.join(items)
        res = self.request_get('/statistics/query?groupId={0}&startDate={1}&endDate={2}&itemName={3}'.format(
            group_id, start_date_str, end_date_str, item_str))
        return sorted([(key, value) for (key, value) in res['data'].iteritems()])

    # Un-documented APIs for admin/operator
    def do_su_login(self, group, user):
        assert group and user
        group_name = group['name'] if isinstance(group, Group) or isinstance(group, EIGroup) else str(group)
        user_name = user['name'] if isinstance(user, User) or isinstance(user, EIUser) else str(user)
        res = self.request_get('/suauth?groupName={0}&userName={1}'.format(group_name, user_name))
        return res['data']

    def do_su_logout(self):
        res = self.request_del('/suauth')
        return res['data']


# -*- coding: utf-8 -*-

import time
import json

from baseapp import BaseApp
from enum import *

class AppPermission(dict):
    """ Structure of an app permission

    Fields:
        category (str): Permission category, refer to the valid values of each apps
        id (str): Permission id, refer to each apps
        level (int): FORBIDDEN (0), VIEW (1), SAVE (2), DELETE (3)
    """
    def __init__(self, category, id, level):
        assert category and id and level >= 0
        super(AppPermission, self).__init__({
            'category': category,
            'id': id,
            'level': level
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['category'], dict_obj['id'], dict_obj['level'])


class AppRole(dict):
    """ Structure of an app role

    Fields:
        name (str): Role name, refer to 'AppRoleName' enum for valid values
        permissions (list): List of AppPermission/dict
    """
    def __init__(self, name, permissions):
        assert name and permissions and isinstance(permissions, list)
        super(AppRole, self).__init__({
            'name': name,
            'permissions': [x if isinstance(x, AppPermission) else AppPermission.from_dict(x) for x in permissions]
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict['permissions'])


class Group(dict):
    """ Structure of a group

    Fields:
        name (str): The group id/name
        displayName (str): The display name
        id (int): The auto id of the group
        createTime (long): The create time
    """
    def __init__(self, name, display_name, id=None, create_time=None):
        assert name and display_name
        super(Group, self).__init__({
            'name': name,
            'displayName': display_name,
            'id': id,
            'createTime': create_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict_obj['displayName'], dict_obj.get('id'), dict_obj.get('createTime'))


class UserRole(dict):
    """ Structure of a user role

    Fields:
        roleName (str): The role of the user for an application. Refer to 'AppRoleName' enum for valid values.
        appId (str): The app id. Refer to 'AppId' enum for valid values
    """
    def __init__(self, role, app_id):
        assert role and app_id
        super(UserRole, self).__init__({
            'roleName': role,
            'appId': app_id
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['roleName'], dict_obj['appId'])


class User(dict):
    """ Structure of a user

    Fields:
        name (str): The login name of the user
        displayName (str): The display name
        password (str): The password
        department (str): The department
        mail (str): The e-mail address
        roles (list): List of UserRole
        id (int): The auto id of the user
        createTime (long): The create time in Epoch (milliseconds)
        lastUpdateTime (long): The last update time in Epoch (milliseconds)
    """
    def __init__(self, name, display_name, password=None, department=None, mail=None, roles=None,
                 id=None, create_time=None, last_update_time=None):
        assert name and display_name
        if roles is None:
            roles = []
        assert isinstance(roles, list)
        if department is None:
            department = ''
        if mail is None:
            mail = ''
        super(User, self).__init__({
            'name': name,
            'displayName': display_name,
            'password': password,
            'department': department,
            'mail': mail,
            'roles': [x if isinstance(x, UserRole) else UserRole.from_dict(x) for x in roles],
            'id': id,
            'createTime': create_time,
            'lastUpdateTime': last_update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict_obj['displayName'], dict_obj.get('password'), dict_obj.get('department'),
                   dict_obj.get('mail'), dict_obj.get('roles'),
                   dict_obj.get('id'), dict_obj.get('createTime'), dict_obj.get('lastUpdateTime'))


class EventCollector(dict):
    """ Structure of Event Collector data source content

    Fields:
        hostName (str): The host/domain name of the data source. Set wildcard (*) to accept data from all domains.
    """
    def __init__(self, hostname):
        assert hostname
        super(EventCollector, self).__init__({
            'hostName': hostname
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['hostName'])


class DataSource(dict):
    """ Structure of a data source

    Fields:
        name (str): Data source id/name
        displayName (str): Data source display name
        appIds (list[str]): The authorized apps in list, refer to 'AppId' enum for valid values
        contentType (str): The data source content type, refer to 'DataSourceContentType' enum for valid values
        id (int): Data source auto id
        groupId (int): Group auto id
        type (str): The data source type (how to get data), refer to 'DataSourceType' enum for valid values
        eventCollector (object): An EventCollector object if type is 'EVENT_COLLECTOR'
        importer (object): An Importer object if type is 'IMPORTER'
        fetch (object): An Fetch object if type is 'FETCH'
        upload (object): An Upload object if type is 'UPLOAD'
    """
    # TODO: importer, fetch, upload not supported yet
    def __init__(self, name, display_name, app_ids, content_type, id=None, group_id=None):
        super(DataSource, self).__init__({
            'name': name,
            'displayName': display_name,
            'appIds': app_ids,
            'contentType': content_type,
            'id': id,
            'groupId': group_id
        })

    def init_event_collector(self, hostname):
        event_collector = EventCollector(hostname)
        self.update({
            'type': DataSourceType.EVENT_COLLECTOR,
            'eventCollector': event_collector
        })

    def init_event_collector_from_dict(self, dict_obj):
        event_collector = EventCollector.from_dict(dict_obj)
        self.update({
            'type': DataSourceType.EVENT_COLLECTOR,
            'eventCollector': event_collector
        })

    @classmethod
    def from_dict(cls, dict_obj):
        new_obj = cls(dict_obj['name'], dict_obj['displayName'], dict_obj['appIds'], dict_obj['contentType'],
                      dict_obj.get('id'), dict_obj.get('groupId'))

        ds_type = dict_obj.get('type')
        if ds_type == DataSourceType.EVENT_COLLECTOR:
            new_obj.init_event_collector_from_dict(dict_obj['eventCollector'])

        return new_obj


class ExporterExtraSchema(dict):
    """ Structure for Extra Schema in Exporter Setting

    Fields:
        name (str): Name/field of the schema
        type (str): Type of the schema, e.g. string, int
        link (str): The tuple key to explode
    """

    def __init__(self, name, type, link):
        assert name and type and link is not None
        super(ExporterExtraSchema, self).__init__({
            'name': name,
            'type': type,
            'link': link
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict_obj['type'], dict_obj['link'])


class ExporterSetting(dict):
    """ Structure for Exporter setting

    Fields:
        enabled (bool): The exporter is enabled or not
        initalConvetTime (long): The initial conversion time in Epoch (milliseconds). Default: now
        tupleKey (str): The tuple key. Default: ptuple
        extraSchema (object): The extra schema
        parsingFormat (str): The input data format. Default: NginxPlusLogParserDriver
        baseSchema (object); The base/default schema
        updateTime (long): The latest data export time in Epoch (milliseconds)

    NOTE:
        initalConvetTime is the legacy typo of "initialConversionTime".
    """
    def __init__(self, enabled, initial_convert_time=None,
                 tuple_key=None, extra_schema=None,
                 parsing_format=None, base_schema=None, update_time=None):
        assert enabled is not None
        if initial_convert_time is None:
            initial_convert_time = long(time.mktime(time.localtime())*1000)
        if not tuple_key:
            tuple_key = 'ptuple'
        if not extra_schema:
            extra_schema = []
        if not parsing_format:
            parsing_format = 'NginxPlusLogParserDriver'
        if not base_schema:
            base_schema = []

        extra_schema_list = [x if isinstance(x, ExporterExtraSchema) else ExporterExtraSchema.from_dict(x) for x in extra_schema]
        extra_schema_str = json.dumps(extra_schema_list)

        base_schema_list = [x if isinstance(x, ExporterExtraSchema) else ExporterExtraSchema.from_dict(x) for x in base_schema]
        base_schema_str = json.dumps(base_schema_list)

        super(ExporterSetting, self).__init__({
            'enabled': enabled,
            'initalConvetTime': long(initial_convert_time),
            'tupleKey': tuple_key,
            'extraSchema': extra_schema_str,
            'parsingFormat': parsing_format,
            'baseSchema': base_schema_str,
            'updateTime': update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        extra_schema = None
        if 'extraSchema' in dict_obj:
            extra_schema = json.loads(dict_obj['extraSchema'])
        base_schema = None
        if 'baseSchema' in dict_obj:
            base_schema = json.loads(dict_obj['baseSchema'])
        return cls(dict_obj['enabled'], dict_obj.get('initalConvetTime'),
                   dict_obj.get('tupleKey'), extra_schema,
                   dict_obj.get('parsingFormat'), base_schema, dict_obj.get('updateTime'))


class EMC2(BaseApp):
    """ Encapsulate Etu Management Center (v2) API """

    __APP_NAME = 'EMC2'
    __HOST = 'emc.online.etunexus.com'
    __API_BASE = '/commsrv/v1'
    __SHIRO_CAS_BASE = '/shiro-cas'

    def __init__(self, cas, host=None, api_base=None, shiro_cas_base=None):
        """ Constructor """

        api_base = api_base if api_base else self.__API_BASE
        super(EMC2, self).__init__(cas, EMC2.__APP_NAME,
                                   api_host=host if host else self.__HOST,
                                   api_base=api_base if api_base else self.__API_BASE,
                                   shiro_cas_base=shiro_cas_base if shiro_cas_base else self.__SHIRO_CAS_BASE)

    # Group #
    def get_groups(self):
        res = self.request_get('/group')
        return [Group.from_dict(x) for x in res]

    def add_group(self, group):
        assert group and isinstance(group, Group)
        res = self.request_post('/group', group)
        return Group.from_dict(res)

    def update_group(self, group):
        assert group and isinstance(group, Group)
        group_id = group['id']
        res = self.request_post('/group/{0}'.format(group_id), group)
        return Group.from_dict(res)

    def del_group(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        assert group_id
        return self.request_del('/group/{0}'.format(group_id))

    # User #
    def me(self):
        res = self.request_get('/user/me')
        return User.from_dict(res)

    def get_users(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/group/{0}/user'.format(group_id))
        return [User.from_dict(x) for x in res]

    def add_user(self, group, user):
        assert group and user and isinstance(user, User)
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_post('/group/{0}/user'.format(group_id), user)
        return User.from_dict(res)

    def update_user(self, user):
        assert user and isinstance(user, User)
        user_id = user['id']
        res = self.request_post('/user/{0}'.format(user_id), user)
        return User.from_dict(res)

    def del_user(self, user):
        assert user
        user_id = user['id'] if isinstance(user, User) else int(user)
        return self.request_del('/user/{0}'.format(user_id))

    def change_user_password(self, user, password):
        assert user and password
        user_id = user['id'] if isinstance(user, User) else int(user)
        params = {
            'password': password
        }
        return self.request_post('/user/{0}/password'.format(user_id), params)

    # Apps #
    def get_apps(self):
        return self.request_get('/app')

    def get_app_permission(self, app_id):
        assert app_id
        res = self.request_get('/app/{0}/permission'.format(app_id))
        assert (res['appId'] == app_id)
        permissions = res['permissions']
        self._logger.debug('app (%s) permissions (%s)' % (app_id, permissions))
        return [AppPermission.from_dict(x) for x in permissions]

    def get_app_roles(self, app_id):
        assert app_id
        res = self.request_get('/app/{0}/role'.format(app_id))
        return [AppRole(x['name'], x['permissions']) for x in res]

    def update_app_role(self, app_id, app_role):
        assert app_id and app_role and isinstance(app_role, AppRole)
        res = self.request_post('/app/{0}/role'.format(app_id), app_role)
        return AppRole.from_dict(res)

    # Data source #
    def get_data_sources(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/group/{0}/data-source'.format(group_id))
        return [DataSource.from_dict(x) for x in res]

    def add_data_source(self, group, data_source):
        assert data_source and isinstance(data_source, DataSource)
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_post('/group/{0}/data-source'.format(group_id), data_source)
        return DataSource.from_dict(res)

    def update_data_source(self, data_source):
        assert data_source and isinstance(data_source, DataSource)
        source_id = data_source['id']
        assert source_id
        res = self.request_post('/data-source/{0}'.format(source_id), data_source)
        return DataSource.from_dict(res)

    def del_data_source(self, data_source):
        assert data_source
        data_source_id = data_source['id'] if isinstance(data_source, DataSource) else int(data_source)
        assert data_source_id
        return self.request_del('/data-source/{0}'.format(data_source_id))

    # Exporter setting #
    def get_exporter_setting(self, data_source):
        assert data_source
        source_id = data_source['id'] if isinstance(data_source, DataSource) else int(data_source)
        res = self.request_get('/data-source/{0}/exporter'.format(source_id))
        return ExporterSetting.from_dict(res)

    def update_exporter_setting(self, data_source, exporter_setting):
        assert data_source and  exporter_setting and isinstance(exporter_setting, ExporterSetting)
        source_id = data_source['id'] if isinstance(data_source, DataSource) else int(data_source)
        assert source_id
        res = self.request_post('/data-source/{0}/exporter'.format(source_id), exporter_setting)
        return ExporterSetting.from_dict(res)



# -*- coding: utf-8 -*-

from baseapp import BaseApp
from emc import Group, DataSource
from enum import *


class AlgInstance(dict):
    """ Structure of an algorithm instance in the recommendation logic.

    AlgInstance is an ABC for all different algorithm structure. The inherited algorithm implementation should be named
    as "Alg_[algId]", e.g. Alg_USER_BASED_CF or Alg_ALS.

    Fields:
        algId (str): The algorithm id, every derived algorithm should have its own id, refer to 'LogicAlgorithmId' for samples
        weight (int): The weight of the algorithm in the linear combination to the recommendation logic result
        setting (object): The setting depend on the concrete algorithm implementation
    """
    def __init__(self, alg_id, weight, setting):
        assert alg_id and weight and setting and isinstance(setting, dict)
        super(AlgInstance, self).__init__({
            'algId': alg_id,
            'weight': weight,
            'setting': setting
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls._create_alg_instance(dict_obj['algId'], dict_obj)

    @classmethod
    def _create_alg_instance(cls, alg_id, dict_obj):
        alg_class_name = 'Alg_{0}'.format(alg_id)
        klass = globals()[alg_class_name]
        return klass.from_dict(dict_obj)


class Alg_USER_BASED_CF(AlgInstance):
    """ Structure of user-based CF algorithm

    Fields:
        (As 'setting' in AlgInstance)
        id (str): 'USER_BASED_CF'
        DATASOURCE (object): A emc.DataSource instance, or at least a dict with 'id' and 'name'
        TIMERANGE (int): The data time range to calculate
        action (list): A list of event actions to calculate, refer to 'EventAction' to samples
    """
    def __init__(self, weight, data_source, time_range, actions):
        assert data_source and time_range and actions and isinstance(actions, list)
        setting = {
            'id': LogicAlgorithmId.USER_BASED_CF,
            'DATASOURCE': {'id': data_source['id'], 'name': data_source['name']},
            'TIMERANGE': time_range,
            'action': actions
        }
        super(Alg_USER_BASED_CF, self).__init__(LogicAlgorithmId.USER_BASED_CF, weight, setting)

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj and dict_obj['setting']
        setting = dict_obj['setting']
        return cls(dict_obj['weight'], setting['DATASOURCE'], setting['TIMERANGE'], setting['action'])


class Alg_ITEM_BASED_CF(AlgInstance):
    """ Structure of item-based CF algorithm

    Fields:
        (As 'setting' in AlgInstance)
        id (str): 'ITEM_BASED_CF'
        DATASOURCE (object): A emc.DataSource instance, or at least a dict with 'id' and 'name'
        TIMERANGE (int): The data time range to calculate
        action (list): A list of event actions to calculate, refer to 'EventAction' to samples
    """
    def __init__(self, weight, data_source, time_range, actions):
        assert data_source and time_range and actions and isinstance(actions, list)
        setting = {
            'id': LogicAlgorithmId.ITEM_BASED_CF,
            'DATASOURCE': {'id': data_source['id'], 'name': data_source['name']},
            'TIMERANGE': time_range,
            'action': actions
        }
        super(Alg_ITEM_BASED_CF, self).__init__(LogicAlgorithmId.ITEM_BASED_CF, weight, setting)

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj and dict_obj['setting']
        setting = dict_obj['setting']
        return cls(dict_obj['weight'], setting['DATASOURCE'], setting['TIMERANGE'], setting['action'])


class Alg_RANKING(AlgInstance):
    """ Structure of ranking algorithm

    Fields:
        (As 'setting' in AlgInstance)
        id (str): 'RANKING'
        DATASOURCE (object): A emc.DataSource instance, or at least a dict with 'id' and 'name'
        TIMERANGE (int): The data time range to calculate
        actionList (list): A list of event actions to calculate, refer to 'EventAction' to samples
        addNonCategoryRec (bool): Generate category-independent recommendation list or not
    """

    def __init__(self, weight, data_source, time_range, actions, gen_non_category_rec=True, delimiter=None):
        assert data_source and time_range and actions
        if delimiter is None:
            delimiter = ','

        setting = {
            'id': LogicAlgorithmId.RANKING,
            'DATASOURCE': {'id': data_source['id'], 'name': data_source['name']},
            'TIMERANGE': time_range,
            'actionList': actions,
            'addNonCategoryRec': str(bool(gen_non_category_rec)).lower(),
            'delimiter': delimiter
        }
        super(Alg_RANKING, self).__init__(LogicAlgorithmId.RANKING, weight, setting)

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj and dict_obj['setting']
        setting = dict_obj['setting']
        return cls(dict_obj['weight'], setting['DATASOURCE'], setting['TIMERANGE'], setting['actionList'],
                   setting.get('addNonCategoryRec'), setting.get('delimiter'))


class UserFilter(dict):
    """ Structure of a user filter setting

    Fields:
        name (str): Name/id
        displayName (str): Display name

        id (int): The auto id
        createTime (long): Create time in Epoch (milliseconds)
        updateTime (long): Update time in Epoch (milliseconds)
    """
    def __init__(self, name, display_name, id=None, create_time=None, update_time=None):
        assert name and display_name
        super(UserFilter, self).__init__({
            'name': name,
            'displayName': display_name,
            'id': id,
            'createTime': create_time,
            'updateTime': update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        assert dict_obj
        return cls(dict_obj['name'], dict_obj['displayName'],
                   dict_obj.get('id'), dict_obj.get('createTime'), dict_obj.get('updateTime'))

class Logic(dict):
    """ Structure of a recommendation logic

    Fields:
        name (str): Name/id
        displayName (str): Display name
        active (bool): Active or not
        numberOfRec (int): Max count of recommendation results
        algType (str): The logic algorithm type, refer to 'LogicAlgType' enum for valid values
        algInstances (list): Algorithm instances in the logic
        userFilter (object): User filter setting
        useLocation (bool): Use location or not
        delegateLogicName (str): The logic(s) for recommendation complementary
        enableUpdating (bool): Enable core engine process or not.

        ## Item Filtering (only for altType==ITEM_BASE)
        enableLastViewedItem (bool): Enable last viewed item recommendation or not (algType=ITEM_BASE)
        itemFilterSrc (str): The item info data source used for filtering
        enableSameCategory (bool): Enable item in same category recommendation or not
        avlItemFilterMode (str): The item avl filter mode, refer to 'LogicAvlItemFilterMode' enum for valid values

        id (int): The auto id
        createTime (long): Create time in Epoch (milliseconds)
        updateTime (long): Update time in Epoch (milliseconds)
    """
    def __init__(self, name, display_name, active, rec_count,
                 alg_type, alg_instances,
                 user_filter=None,
                 use_location=False, enable_last_viewed_item=False,
                 complementary_logics=None, enable_updating=True,
                 item_filter_src=None, enable_same_category=False, avl_item_filter_mode=LogicAvlItemFilterMode.DISABLED,
                 id=None, create_time=None, update_time=None):
        assert name and display_name and rec_count
        assert active and isinstance(active, bool)
        assert alg_type and alg_instances and isinstance(alg_instances, list)
        if user_filter is not None:
            assert isinstance(user_filter, UserFilter)
        assert isinstance(use_location, bool)
        assert isinstance(enable_last_viewed_item, bool)
        assert isinstance(enable_same_category, bool)
        super(Logic, self).__init__({
            'name': name,
            'displayName': display_name,
            'active': active,
            'numberOfRec': rec_count,
            'algType': alg_type,
            'algInstances': [x if isinstance(x, AlgInstance) else AlgInstance.from_dict(x) for x in alg_instances],
            'userFilter': user_filter,
            'useLocation': use_location,
            'enableLastViewedItem': enable_last_viewed_item,
            'delegateLogicName': complementary_logics,
            'enableUpdating': enable_updating,

            'itemFilterSrc': item_filter_src,
            'enableSameCategory': enable_same_category,
            'avlItemFilterMode': avl_item_filter_mode,

            'id': id,
            'createTime': create_time,
            'updateTime': update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['name'], dict_obj['displayName'], dict_obj['active'], dict_obj['numberOfRec'],
                   dict_obj['algType'], dict_obj['algInstances'],
                   dict_obj.get('userFilter'),
                   dict_obj['useLocation'], dict_obj['enableLastViewedItem'],
                   dict_obj.get('delegateLogicName'), dict_obj['enableUpdating'],
                   dict_obj.get('itemFilterSrc'), dict_obj['enableSameCategory'], dict_obj['avlItemFilterMode'],
                   dict_obj.get('id'), dict_obj.get('createTime'), dict_obj.get('updateTime'))


class Campaign(dict):
    """ Structure for a recommendation campaign

    Fields:
        name (str): Campaign name
        displayName (str): Campaign display name
        startTime (long): Campaign start time in Epoch (milliseconds)
        endTime (long): Campaign end time in Epoch (milliseconds)
        logics (list): A list of 'Logic' instances
        id (int): The auto id
        createTime (long): Campaign create time in Epoch (milliseconds)
        updateTime (long): Campaign update time in Epoch (milliseconds)
    """
    def __init__(self, group_id, name, display_name, start_time, end_time, logics=None, id=None, create_time=None, update_time=None):
        assert group_id and name and display_name
        assert 0 <= start_time and 0 <= end_time and start_time < end_time
        if logics is None:
            logics = []
        super(Campaign, self).__init__({
            'groupId': group_id,
            'name': name,
            'displayName': display_name,
            'startTime': start_time,
            'endTime': end_time,
            'logics': [x if isinstance(x, Logic) else Logic.from_dict(x) for x in logics],
            'id': id,
            'createTime': create_time,
            'updateTime': update_time
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls(dict_obj['groupId'], dict_obj['name'], dict_obj['displayName'],
                   dict_obj['startTime'], dict_obj['endTime'],
                   dict_obj['logics]'],
                   dict_obj.get('id'), dict_obj.get('createTime'), dict_obj.get('updateTime'))


class Layout(dict):
    """ Structure for recommendation layout generator

    Fields:
        dataSrcName (str): The item info data source name/id
        loName: The layout name/id
        loTitle": "首頁推薦",
        loTitleAlign": "center",
        loItemCnt": 10,
        loItemWidth": 300,
        loItemHeight": 240,
        loItemMargin": 20,
        loBGColor": "rgba(255,255,255,1)",
        loFGColor": "rgba(0,0,0,1)",
        loCol": 5,
        loRow": 2,
        loFontSize": 16

        loID (int): The auto id
    """

    def __init__(self):
        super(Layout, self).__init__({
        })

    @classmethod
    def from_dict(cls, dict_obj):
        return cls()


class ER3(BaseApp):
    """ Encapsulate Etu Recommender (v3) API """

    __APP_NAME = 'ER3'
    __HOST = 'erhome.online.etunexus.com'
    __API_BASE = '/ER/V3/recsrv/v1'
    __SHIRO_CAS_BASE = '/ER/V3/shiro-cas'

    def __init__(self, cas, host=None, api_base=None, shiro_cas_base=None):
        """ Constructor """

        api_base = api_base if api_base else self.__API_BASE
        super(ER3, self).__init__(cas, ER3.__APP_NAME,
                                  api_host=host if host else self.__HOST,
                                  api_base=api_base if api_base else self.__API_BASE,
                                  shiro_cas_base=shiro_cas_base if shiro_cas_base else self.__SHIRO_CAS_BASE)

    # Logic #
    def get_logics(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/group/{0}/logic'.format(group_id))
        return [Logic.from_dict(x) for x in res]

    def add_logic(self, group, logic):
        assert group and logic and isinstance(logic, Logic)
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_post('/group/{0}/logic'.format(group_id), logic)
        return Logic.from_dict(res)

    def update_logic(self, logic):
        assert logic and isinstance(logic, Logic)
        logic_id = logic['id']
        assert logic_id
        res = self.request_post('/logic/{0}'.format(logic_id), logic)
        return Logic.from_dict(res)

    def del_logic(self, logic):
        assert logic
        logic_id = logic['id'] if isinstance(logic, Logic) else logic
        assert logic_id
        return self.request_del('/logic/{0}'.format(logic_id))

    # Campaign #
    def get_campaigns(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/group/{0}/campaign'.format(group_id))
        return [Campaign.from_dict(x) for x in res]

    def add_campaign(self, group, campaign):
        assert group and campaign and isinstance(campaign, Campaign)
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_post('/group/{0}/campaign', campaign)
        return Campaign.from_dict(res)

    def update_campaign(self, campaign):
        assert campaign and isinstance(campaign, Campaign)
        campaign_id = campaign['id']
        assert campaign_id
        res = self.request_post('/campaign/{0}'.format(campaign_id), campaign)
        return Campaign.from_dict(res)

    def del_campaign(self, campaign):
        assert campaign
        campaign_id = campaign['id'] if isinstance(campaign, Campaign) else int(campaign)
        assert campaign_id
        return self.request_del('/campaign/{0}'.format(campaign_id))

    # User filter #
    def get_user_filters(self, group):
        assert group
        group_id = group['id'] if isinstance(group, Group) else int(group)
        res = self.request_get('/group/{0}/userfilter'.format(group_id))
        return [UserFilter.from_dict(x) for x in res]

    def add_user_filter(self, group_id, user_filter, file_path):
        assert group_id and user_filter and isinstance(user_filter, UserFilter)
        res = self.request_upload('/group/{0}/userfilter'.format(group_id), user_filter, file_path)
        return UserFilter.from_dict(res)

    def update_user_filter(self, user_filter, file_path):
        assert user_filter and isinstance(user_filter, UserFilter)
        user_filter_id = user_filter['id']
        assert user_filter_id
        res = self.request_upload('/userfilter/{0}'.format(user_filter_id), user_filter, file_path)
        return UserFilter.from_dict(res)

    def del_user_filter(self, user_filter):
        assert user_filter
        user_filter_id = user_filter['id'] if isinstance(user_filter, UserFilter) else int(user_filter)
        assert user_filter_id
        return self.request_del('/userfilter/{0}'.format(user_filter_id))

    # Layout #
    def get_layouts(self, group_id):
        assert group_id
        res = self.request_get('/group/{0}/layout'.format(group_id))
        return [Layout.from_dict(x) for x in res]

    def add_layout(self, group_id, layout):
        assert group_id and layout and isinstance(layout, Layout)
        res = self.request_post('/group/{0}/layout'.format(group_id), layout)
        return Layout.from_dict(res)

    def update_layout(self, layout):
        assert layout and isinstance(layout, Layout)
        layout_id = layout['id']
        assert layout_id
        res = self.request_post('/layout/{0}'.format(layout_id), layout)
        return Layout.from_dict(res)

    def del_layout(self, layout):
        assert layout
        layout_id = layout['id'] if isinstance(layout, Layout) else int(layout)
        assert layout_id
        return self.request_del('/layout/{0}'.format(layout_id))



from setuptools import setup, find_packages

setup(name='etunexus',
      version='0.5.1',
      description='Etu Nexus API',
      url='http://www.etunexus.com/',
      author='Etu Corporation',
      author_email='indichen@etusolution.com',
      license='GPL',
      packages=find_packages(),
      install_requires=[
            'MultipartPostHandler',
      ],
      zip_safe=False)